--- common/objects.c.orig	2008-07-30 20:02:58.000000000 +0200
+++ common/objects.c	2008-07-30 21:44:02.000000000 +0200
@@ -57,6 +57,9 @@
 
 skiplist *object_skiplists[NUM_OBJECT_SKIPLISTS];
 
+void create_orphan_hosts_hostgroup();
+void create_orphan_services_servicegroup();
+
 
 #ifdef NSCORE
 int __nagios_object_structure_version=CURRENT_OBJECT_STRUCTURE_VERSION;
@@ -85,6 +88,10 @@
 		return ERROR;
 #endif
 
+        /* create a new hostgroup with orphaned hosts and a servicegroup with orphaned services */
+	create_orphan_hosts_hostgroup();
+	create_orphan_services_servicegroup();
+
 	return result;
         }
 
@@ -4106,3 +4113,64 @@
 	return OK;
         }
 
+
+
+/* Creates a hostgroup with hosts which are not a member of any hostgroup */
+void create_orphan_hosts_hostgroup() {
+
+    hostgroup *orphan_hostgroup;
+    host *temp_host;
+    hostgroup *temp_hostgroup;
+
+    orphan_hostgroup = add_hostgroup("orphans","Orphan Hosts","","","");
+
+    if (orphan_hostgroup) {
+        for (temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next) {
+            int found = 0;
+            for (temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next) {
+                hostsmember *member;
+                for (member=temp_hostgroup->members;member!=NULL;member=member->next) {
+                    // Remember we have found this host
+                    if (strcmp(member->host_name,temp_host->name)==0)
+                        found++;
+                }
+            }
+            // If we didn't find this host in any hostgroup, this is an orphaned host
+            if (found==0) {
+                add_host_to_hostgroup(orphan_hostgroup,temp_host->name);
+            }
+        }
+    }
+
+}
+
+
+/* Creates a servicegroup with services which are not a member of any servicegroup */
+void create_orphan_services_servicegroup() {
+
+    servicegroup *orphan_servicegroup;
+    service *temp_service;
+    servicegroup *temp_servicegroup;
+
+    orphan_servicegroup = add_servicegroup("orphans","Orphan Services","","","");
+
+    if (orphan_servicegroup) {
+        for (temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next) {
+            int found = 0;
+            for (temp_servicegroup=servicegroup_list;temp_servicegroup!=NULL;temp_servicegroup=temp_servicegroup->next) {
+                servicesmember *member;
+                for (member=temp_servicegroup->members;member!=NULL;member=member->next) {
+                    // Remember we have found this host
+                    if (strcmp(member->service_description,temp_service->description)==0)
+                        found++;
+                }
+            }
+            // If we didn't find this service in any servicegroup, this is an orphaned service
+            if (found==0) {
+                add_service_to_servicegroup(orphan_servicegroup,temp_service->host_name,temp_service->description);
+            }
+        }
+    }
+
+}
+
